--
-- Copyright (C) 1988-1999 Altera Corporation
-- Any megafunction design, and related net list (encrypted or decrypted),
-- support information, device programming or simulation file, and any other
-- associated documentation or information provided by Altera or a partner
-- under Altera's Megafunction Partnership Program may be used only to
-- program PLD devices (but not masked PLD devices) from Altera.  Any other
-- use of such megafunction design, net list, support information, device
-- programming or simulation file, or any other related documentation or
-- information is prohibited for any other purpose, including, but not
-- limited to modification, reverse engineering, de-compiling, or use with
-- any other silicon devices, unless such use is explicitly licensed under
-- a separate agreement with Altera or a megafunction partner.  Title to
-- the intellectual property, including patents, copyrights, trademarks,
-- trade secrets, or maskworks, embodied in any such megafunction design,
-- net list, support information, device programming or simulation file, or
-- any other related documentation or information provided by Altera or a
-- megafunction partner, remains with Altera, the megafunction partner, or
-- their respective licensors.  No other licenses, including any licenses
-- needed under any third party's intellectual property, are provided herein.
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Version 1.00 Date: 11/15/99

-- 09/20/99:
-- changed ALTPLL name to ALTCLKLOCK
-- changed mult_factor_clock0 to clock0_boost
-- changed mult_factor_clock1 to clock1_boost
-- changed divide_factor_clock0 to clock0_divide
-- changed divide_factor_clock1 to clock1_divide
-- added lvds_rx and lvds_tx megafuntions
-- 
-- 09/28/99:
-- lvds receiver:
-- 	changed input_frequency to inclock_frequency 
-- 	changed rx_clkin to rx_inclock
-- 	changed rx_outclk to rx_outclock
--    corrected the x7 loop index problem
-- lvds transmitter:
-- 	changed input_frequency to inclock_frequency 
-- 	changed tx_clkin to tx_inclock
-- 	changed tx_outclk to tx_outclock
-- 	changed synch_clock_in  to sync_inclock
--
-- 10/05/99:
-- Changed inclock_frequency to inclock_period for ALTCLKLOCK, ALTLVDS_RX,
-- and ALTLVDS_TX
--
-- 10/19/99:
-- SPR65264: Corrected CAM initialization from *.hex; i.e. fix to
--           function hex_to_stdlogicarray (s: string).
--
-- 10/22/99:
-- SPR65544: Corrected SINGLE mode operation when outputreg is clocked
--           on either "INCLOCK" or "OUTCLOCK".
--
-- 10/27/99: 
-- SPR65010: Fixed lvds reciever and transmitter number of cycles to output.
--           Converted integer and positive types to natural to work with
--           Quartus Wizzard. 
--           Changed altclklock outclock_phase_shift type from real to natural
--           which defaults to ps.

-- 11/15/99: Fixed the lvds deskew according to the new spec.
-- 11/15/99: Made altclklock locked output sensetive to clock variations

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use std.textio.all;

entity altcam is
   generic
      ( width:            natural ;
        widthad:          natural ;
        numwords:         natural := 0;
        lpm_file:         string := "UNUSED";
        lpm_filex:        string := "UNUSED";
        match_mode:       string := "MULTIPLE";
        output_reg:       string := "UNREGISTERED";
        output_aclr:      string := "OFF";
        pattern_reg:      string := "INCLOCK";
        pattern_aclr:     string := "ON";
        wraddress_aclr:   string := "ON";
        wrx_reg:          string := "INCLOCK";
        wrx_aclr:         string := "ON";
        wrcontrol_aclr:   string := "OFF" );
        
   port
      ( pattern:    in std_logic_vector(width-1 downto 0);
        wrx:        in std_logic_vector(width-1 downto 0) := (others => '0');
        wrxused:    in std_logic := '0';
        wrdelete:   in std_logic := '0';
        wraddress:  in std_logic_vector(widthad-1 downto 0);
        wren:       in std_logic;
        inclock:    in std_logic;
        inclocken:  in std_logic := '1';
        inaclr:     in std_logic := '0';
        mstart:     in std_logic := '0';
        mnext:      in std_logic := '0';
        outclock:   in std_logic := '0';
        outclocken: in std_logic := '1';
        outaclr:    in std_logic := '0';
        maddress:   out std_logic_vector(widthad-1 downto 0);
        mbits:      out std_logic_vector(numwords-1 downto 0);
        mfound:     out std_logic;
        mcount:     out std_logic_vector(widthad-1 downto 0);
        rdbusy:     out std_logic;
        wrbusy:     out std_logic );

    type lpm_memory is array 
        ((2**widthad)-1 downto 0) of std_logic_vector(width-1 downto 0);
    function int_to_str( value : integer ) return string is
    variable ivalue,index : integer;
    variable digit : integer;
    variable line_no: string(8 downto 1) := "        ";  
    begin
    ivalue := value;
    index := 1;   
    while (ivalue > 0 ) loop
       digit := ivalue MOD 10;
       ivalue := ivalue/10;
       case digit is
           when 0 =>
              line_no(index) := '0';
           when 1 =>
              line_no(index) := '1';
           when 2 =>
              line_no(index) := '2';
           when 3 =>
              line_no(index) := '3';
           when 4 =>
              line_no(index) := '4';
           when 5 =>
              line_no(index) := '5';
           when 6 =>
              line_no(index) := '6';
           when 7 =>
              line_no(index) := '7';
           when 8 =>
              line_no(index) := '8';
           when 9 =>
              line_no(index) := '9';
           when others =>
              ASSERT FALSE
              REPORT "Illegal number!"
              SEVERITY ERROR;
       end case;
       index := index + 1;
    end loop;
    return line_no;
 end;

    function hex_str_to_int( str : string ) return integer is
    variable len : integer := str'length;
    variable ivalue : integer := 0;
    variable digit : integer;
    begin
    for i in len downto 1 loop
       case str(i) is
          when '0' =>
                digit := 0;
          when '1' =>
             digit := 1;
          when '2' =>
             digit := 2;
          when '3' =>
             digit := 3;
          when '4' =>
             digit := 4;
          when '5' =>
             digit := 5;
          when '6' =>
             digit := 6;
          when '7' =>
             digit := 7;
          when '8' =>
             digit := 8;
          when '9' =>
             digit := 9;
          when 'A' =>
             digit := 10;
          when 'a' =>
             digit := 10;
          when 'B' =>
             digit := 11;
          when 'b' =>
             digit := 11;
          when 'C' =>
             digit := 12;
          when 'c' =>
             digit := 12;
          when 'D' =>
             digit := 13;
          when 'd' =>
             digit := 13;
          when 'E' =>
             digit := 14;
          when 'e' =>
             digit := 14;
          when 'F' =>
             digit := 15;
          when 'f' =>
             digit := 15;
          when others =>
           ASSERT FALSE
           REPORT "Illegal character "&  str(i) & "in Intel Hex File! "
           SEVERITY ERROR;
   end case;
   ivalue := ivalue * 16 + digit;
   end loop;
   return ivalue;
 end;

   procedure Shrink_line(L : inout LINE; pos : in integer)
   is
           subtype nstring is string(1 to pos);
           variable stmp : nstring;
   begin
       if pos >= 1 then
           read(l,stmp);
       end if;
   end;


function hex_to_stdlogicarray (s: string) return lpm_memory is
   variable mem_data : lpm_memory;
   variable mem_data_tmp : integer := 0;
   variable i,j,k,lineno: integer := 0;
   variable buf: line ;
   variable booval: boolean ;
   FILE mem_data_file: TEXT IS IN s;
   variable base, byte, rec_type, datain, checksum: string(2 downto 1);
   variable startadd: string(4 downto 1);
   variable ibase: integer := 0;
   variable ibyte: integer := 0;
   variable istartadd: integer := 0;
   variable check_sum_vec, check_sum_vec_tmp: std_logic_vector(7 downto 0);
begin
         WHILE NOT ENDFILE(mem_data_file) loop
            booval := true;
            READLINE(mem_data_file, buf);
            lineno := lineno + 1;
            check_sum_vec := (OTHERS => '0');
            if (buf(buf'LOW) = ':') then  -- check for start of record char ':'
               i := 1;
               shrink_line(buf, i);
               READ(L=>buf, VALUE=>byte, good=>booval); -- read length of record (2 hex chars)
               if not (booval) then
                  ASSERT FALSE
                  REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format!"
                  SEVERITY ERROR;
               end if;
               ibyte := hex_str_to_int(byte);
               check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(ibyte, 8));
               READ(L=>buf, VALUE=>startadd, good=>booval); -- read start/load address (4 hex chars)
               if not (booval) then 
                  ASSERT FALSE
                  REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format! "
                  SEVERITY ERROR;
               end if;
               istartadd := hex_str_to_int(startadd);
               byte(2) := startadd(4);
               byte(1) := startadd(3);
               check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(hex_str_to_int(byte), 8));
               byte(2) := startadd(2);
               byte(1) := startadd(1);
               check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(hex_str_to_int(byte), 8));
               READ(L=>buf, VALUE=>rec_type, good=>booval);  -- read record type (2 hex chars)
               if not (booval) then
                  ASSERT FALSE
                  REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format! "
                  SEVERITY ERROR;
               end if;
               check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(hex_str_to_int(rec_type), 8));
            else
               ASSERT FALSE
               REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format! "
               SEVERITY ERROR;
            end if;
	    case rec_type is
               when "00"=>  -- Data record
                  i := 0;
                  k := width/8;
                  if ( (width MOD 8) /= 0 ) then
                       k := k + 1; 
                  end if;
                  -- k = no. of bytes per CAM entry.
                  while( i < ibyte ) loop
                     mem_data_tmp := 0;
                     for j in 1 to k loop
                        READ(L=>buf, VALUE=>datain,good=>booval); -- read in data a byte (2 hex chars) at a time.
                        if not (booval) then
                           ASSERT FALSE
                           REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format! "
                           SEVERITY ERROR;
                        end if;
                        check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(hex_str_to_int(datain), 8));
                        mem_data_tmp := mem_data_tmp * 256 + hex_str_to_int(datain);
                     end loop;
                     i := i + k;
                     mem_data(ibase + istartadd) := CONV_STD_LOGIC_VECTOR(mem_data_tmp, width);
                     istartadd := istartadd + 1;
                   end loop;
               when "01"=>
                  exit;
               when "02"=>
                  ibase := 0;
                  if (ibyte /= 2) then
                           ASSERT FALSE
                           REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format for record type 02! "
                           SEVERITY ERROR;
                  end if;
                  for i in 0 to (ibyte-1) loop
                     READ(L=>buf, VALUE=>base,good=>booval);
                     if not (booval) then
                        ASSERT FALSE
                        REPORT "[Line "& int_to_str(lineno) & "]:Illegal Intel Hex Format! "
                        SEVERITY ERROR;
                     end if;
                     ibase := ibase * 256 + hex_str_to_int(base);
                     check_sum_vec := unsigned(check_sum_vec) + unsigned(CONV_STD_LOGIC_VECTOR(hex_str_to_int(base), 8));
                  end loop;
                  ibase := ibase * 16;  -- because std load addr is 2 bytes.
               when OTHERS =>
                  ASSERT FALSE
                  REPORT "[Line "& int_to_str(lineno) & "]:Illegal record type in Intel Hex File! "
                  SEVERITY ERROR;
            end case;
            READ(L=>buf, VALUE=>checksum,good=>booval);
            if not (booval) then
               ASSERT FALSE
               REPORT "[Line "& int_to_str(lineno) & "]:Checksum is missing! "
               SEVERITY ERROR;
            end if;

            check_sum_vec := unsigned(not (check_sum_vec)) + 1 ;
            check_sum_vec_tmp := CONV_STD_LOGIC_VECTOR(hex_str_to_int(checksum),8);

            if (check_sum_vec /= check_sum_vec_tmp) then
               ASSERT FALSE
               REPORT "[Line "& int_to_str(lineno) & "]:Incorrect checksum!"
               SEVERITY ERROR;
            end if;
         end loop;
         return mem_data;
end;

end altcam;	

architecture behave of altcam is

signal pattern_rgd: std_logic_vector(width-1 downto 0); -- registered input
signal pattern_int: std_logic_vector(width-1 downto 0); -- internal input
signal wrx_rgd: std_logic_vector(width-1 downto 0); -- registered input
signal wrxused_rgd : std_logic;
signal wrxused_int : std_logic;
signal wrx_int: std_logic_vector(width-1 downto 0); -- internal input
signal wraddress_rgd: std_logic_vector(widthad-1 downto 0); -- registered input
signal wren_rgd: std_logic; -- registered input
signal maddress_rgd: std_logic_vector(widthad-1 downto 0); --registered output
signal maddress_int: std_logic_vector(widthad-1 downto 0) := (others => 'U'); -- internal output
signal mbits_rgd: std_logic_vector(numwords-1 downto 0); -- registered output
signal mbits_int: std_logic_vector(numwords-1 downto 0) := (others => '0'); -- internal output
signal mfound_rgd: std_logic; -- registered output
signal matchfound_clk1: std_logic; -- registered output
signal mfound_int: std_logic := '0'; -- internal output
signal mcount_rgd: std_logic_vector(widthad-1 downto 0); -- registered output
signal mcount_int: std_logic_vector(widthad-1 downto 0) := (others => '0'); -- internal output
signal cam_array: lpm_memory;
signal x_array: lpm_memory;
signal mstart_last_value : std_logic;
signal first_read_clock : boolean := false;
signal second_read_clock : boolean := false;
signal next_read_clock : boolean := false;
signal write_clock : std_logic;
signal get_next_match : std_logic := '0';
signal wrbusy_int : std_logic := '0';
signal write_start : std_logic;
signal rdbusy_int : std_logic := '0';
signal cam_init: integer := 1;
signal write_start_rgd: std_logic := '0';
signal write_start_1: std_logic := '0';
signal mstart_rgd1: std_logic := '0';
signal mstart_rgd2: std_logic := '0';
signal print: integer := 1;
signal write_incomplete: std_logic := '0';
signal rdbusy_delayed : std_logic;
signal wrdelete_delayed : std_logic;
signal write0: boolean := true;
signal write1: boolean := false;
signal writex: boolean := false;
signal write0_done: boolean := false;
signal write1_done: boolean := false;
signal writex_done: boolean := false;
signal outclock_int: std_logic;  -- internal outclock
signal outaclr_int: std_logic;   -- internal outaclr

begin

msg: process (wraddress_rgd, wrdelete, wrdelete_delayed, wren_rgd, 
              wrxused_int, pattern_int, mstart_rgd1)
begin
      if wren_rgd'event and wren_rgd = '0' and write_incomplete = '1' then
        assert false
           report "Insufficient write cycle time, write failed! "
           severity warning;
      elsif pattern_int'event and write_incomplete = '1' then
        assert false
           report "Insufficient pattern hold time, write failed! "
           severity warning;
      elsif wraddress_rgd'event and write_incomplete = '1' then
        assert false
           report "Insufficient address hold time, write failed! "
           severity warning;
      elsif wrdelete_delayed'event and wrdelete_delayed = '0' and 
            write_incomplete = '1' then
        assert false
           report "Insufficient delete cycle time, delete failed! "
           severity warning;
      elsif wrdelete'event and wrdelete = '1' and 
           write_incomplete = '1' then
        assert false
           report "Insufficient write cycle time, write failed! "
           severity warning;
      elsif wrxused_int'event and write_incomplete = '1' and 
            wrdelete = '0' then
        assert false
           report "wrxused signal changed during write! "
           severity warning;
      elsif mstart_rgd1'event and write_incomplete = '1' and 
            mstart_rgd1 = '1' then
        assert false
           report "Incorrect read attempt during write! "
           severity warning;
      end if;
   if pattern_int'event then
      if rdbusy_delayed = '1' then
        assert false
           report "Insufficient read time, read failed! "
           severity warning;
      elsif rdbusy_delayed = '0' and mnext = '1' then
        assert false
           report "Illegal pattern change during read, read failed! "
           severity warning;
      end if;
   end if;
  
end process msg;

-- Evaluate parameters
pattern_int <= pattern when pattern_reg = "UNREGISTERED" else pattern_rgd;
wrx_int     <= wrx when wrx_reg = "UNREGISTERED" else wrx_rgd;
wrxused_int <= wrxused when wrx_reg = "UNREGISTERED" else wrxused_rgd;
maddress    <= maddress_int when output_reg = "UNREGISTERED" else maddress_rgd;
mbits       <= mbits_int when output_reg = "UNREGISTERED" else mbits_rgd;
mfound      <= mfound_int when output_reg = "UNREGISTERED" else mfound_rgd;
mcount      <= mcount_int when output_reg = "UNREGISTERED" else mcount_rgd;
wrbusy      <= wrbusy_int;
rdbusy      <= rdbusy_int;
outclock_int <= outclock when output_reg = "OUTCLOCK" else inclock;
outaclr_int  <= outaclr when output_reg = "OUTCLOCK" else inaclr;

-- Generate write clock
write_clock <= (not inclock) and wren_rgd;
wrdelete_delayed <= wrdelete after 2 ns;
rdbusy_delayed <= rdbusy_int after 2 ns;

-- Evaluate registered signals
inputreg: process (inaclr, inclock)

    variable count : natural := 0;
    variable index : natural :=0;
    variable next_search : natural := 0;

begin
   if inaclr = '1' then
	   if pattern_aclr = "ON" then 
         pattern_rgd <= (others => '0'); 
      end if;
	   if wrx_aclr = "ON" then 
         wrx_rgd <= (others => '0'); 
	      wrxused_rgd <= '0';
      end if;
	   if wraddress_aclr = "ON" then 
         wraddress_rgd <= (others => '0'); 
      end if;
	   if wrcontrol_aclr = "ON" then 
         wren_rgd <= '0'; 
      end if;
	elsif inclock'event and inclock = '1' and inclocken = '1' then
      pattern_rgd <= pattern;
      wrx_rgd <= wrx;
      wrxused_rgd <= wrxused;
      wraddress_rgd <= wraddress;
      wren_rgd <= wren;
      write_start_rgd <= write_start;
      write_incomplete <= wrbusy_int;
 
-- read
     if wren_rgd = '0' then
	   if first_read_clock = true or match_mode = "SINGLE" then 
         count := 0; 
         index := 0;
         next_search := 0;
         second_read_clock <= true;
         next_read_clock <= false;
   --    mfound_int <= '0'; -- Force mfound to 0 when count is 0.
         mbits_int <= (others => '0');
      end if;
	   if match_mode = "SINGLE" or match_mode = "FASTMULTIPLE" or 
         second_read_clock = true or get_next_match = '1' then
         second_read_clock <= false;
      if get_next_match = '1' then 
         index := next_search; 
      end if;
      maddr: for i in index to 2**widthad -1 loop
          if get_next_match = '1' and next_search = 2**widthad then
		       mfound_int <= '0';
		       exit maddr;
	       end if;
        mword:    for j in 0 to width-1 loop
		    if (((x_array(i)(j) = '0') and 
             (cam_array(i)(j) = pattern_int(j))) or 
             ((x_array(i)(j) = '1') and 
             (cam_array(i)(j) = '0'))) then 
		       if j = width-1 then 
		          mfound_int <= '1';
		          count := count + 1;
                mbits_int(i) <= '1';
                maddress_int <= 
                ieee.std_logic_arith.conv_std_logic_vector(i, widthad);
                if i < 2**widthad  then
                   next_search := i+1;
		             next_read_clock <= true;
                end if;
		          exit maddr;
		       end if;
		    else 
             exit mword;
          end if;
	     end loop mword;
        if (i = (2 **widthad) -1) and count = 0 then
          mfound_int <= '0'; 
        end if;
	   end loop maddr;
	   end if;
      mcount_int <= 
      ieee.std_logic_arith.conv_std_logic_vector(count, widthad);
   end if;
   end if; -- read
end process inputreg;

outputreg: process (outaclr_int, outclock_int)
begin
  if output_reg = "OUTCLOCK" or output_reg = "INCLOCK" then
    if outaclr_int = '1' then
       if output_aclr = "ON" then 
          maddress_rgd <= (others => '0');
	  mbits_rgd <= (others => '0');
	  mfound_rgd <= '0';
	  mcount_rgd <= (others => '0');
       end if;
    elsif outclock_int'event and outclock_int = '1' and outclocken = '1' then
         mstart_rgd1 <= mstart;
         mstart_rgd2 <= mstart_rgd1;
         maddress_rgd <= maddress_int;
         mbits_rgd <= mbits_int;
         mfound_rgd <= mfound_int;
         mcount_rgd <= mcount_int;
    end if;
  end if;
end process outputreg;

write_busy_control: process(wren_rgd, wraddress_rgd, write_start_rgd) 
begin
   if wren_rgd'event and wren_rgd = '1' then 
      wrbusy_int <= '1' ;
   elsif wraddress_rgd'event and wren_rgd = '1' then
      wrbusy_int <= '1' ;
   elsif write_start_rgd'event and write_start_rgd = '1' then
      wrbusy_int <= '0' ;
   end if;
end process write_busy_control;

write_control: process(write0_done, write1_done, writex_done, wrbusy_int)
begin
   if wrbusy_int'event and wrbusy_int = '1' then
      write0 <= true;
   elsif wrbusy_int'event and wrbusy_int = '0' then 
      write0 <= false;
   elsif write0_done'event and write0_done = true then
      write1 <= true;
   elsif write1_done'event and write1_done = true then
      if wrxused_int = '1' then
         writex <= true;
      else 
         writex <= false;
      end if;
      write1 <= false;
   elsif writex_done'event and writex_done = true then
      write1 <= false;
      writex <= false;
   end if;
end process write_control;

write: process(write_clock, wrdelete, pattern_int, wrx_int, wraddress_rgd)
begin
   if cam_init = 1  then	
      if lpm_file = "UNUSED" then
         for i in 0 to (2**widthad)-1 loop
            cam_array(i) <= (others => '1');
            x_array(i) <= (others => '1');
         end loop;
      elsif lpm_filex = "UNUSED" then
	      cam_array <= hex_to_stdlogicarray(lpm_file);
         for i in 0 to (2**widthad)-1 loop
	         x_array(i) <= (others => '0');
         end loop;
      else
         cam_array <= hex_to_stdlogicarray(lpm_file);
         x_array <= hex_to_stdlogicarray(lpm_filex);
      end if;
   end if;
   cam_init <= 0;

   if write_clock = '1' then
     if wrdelete = '0' then
        if wrxused_int = '1' then
--write_zeros:
 	        if write0 = true then
              for i in 0 to width -1 loop
                 if pattern_int(i) = '0' then
            cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                    <= '0';
            x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                    <= '0';
                 end if;
              end loop;
              write0_done <= true;
              write1_done <= false;
              writex_done <= false;
  	     end if;

--write_ones:
 	     if write1 = true then
           for i in 0 to width -1 loop
              if pattern_int(i) = '1' then
            cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '1';
            x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '0';
              end if;
           end loop;
           write1_done <= true;
           write0_done <= false;
           writex_done <= false;
  	     end if;

--write_x:
 	     if writex = true then
           for i in 0 to width -1 loop
         x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
              <= wrx_int(i);
              if pattern_int(i) = '1' and wrx_int(i) = '1' then
            cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '0';
              end if;
           end loop;
           writex_done <= true;
           write0_done <= false;
           write1_done <= false;
  	     end if;
        if wrbusy_int = '1' then
           write_start_1 <= '1';
           write_start <= write_start_1;
        else
           write_start_1 <= '0';
           write_start <= '0';
        end if;
     else
--write_zeros:
 	     if write0 = true then
           for i in 0 to width -1 loop
              if pattern_int(i) = '0' then
            cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '0';
            x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '0';
              end if;
           end loop;
           write0_done <= true;
           write1_done <= false;
           writex_done <= false;
  	     end if;

--write_ones:
 	     if write1 = true then
           for i in 0 to width -1 loop
              if pattern_int(i) = '1' then
            cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '1';
            x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
                 <= '0';
              end if;
           end loop;
           write1_done <= true;
           write0_done <= false;
           writex_done <= false;
  	     end if;

        if wrbusy_int = '1' then
           write_start <= '1';
        else
           write_start <= '0';
        end if;
     end if;
     else
--write_ones:
        for i in 0 to width -1 loop
         cam_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
             <= '1';
        end loop;
        write1_done <= true;
        write0_done <= false;
        writex_done <= false;

--write_x:
 	     if writex = true then
           for i in 0 to width -1 loop
         x_array(ieee.std_logic_unsigned.conv_integer(wraddress_rgd))(i)
              <= '1';
           end loop;
           writex_done <= true;
           write0_done <= false;
           write1_done <= false;
  	     end if;
        if wrbusy_int = '1' then
           write_start <= '1';
        else
           write_start <= '0';
        end if;
     end if;
   end if;
end process write;

read_control: process(mstart, mstart_rgd1, mstart_rgd2, mnext)
begin
    if mstart'event and mstart = '1' then
	     first_read_clock <= true;
    elsif mstart_rgd1'event and mstart_rgd1 = '1' then
	     first_read_clock <= false;
        if match_mode = "SINGLE" or match_mode = "FASTMULTIPLE" then
           rdbusy_int <= '0' ;
	else
	   rdbusy_int <= '1';  -- must be "MULTIPLE" mode
        end if;
    elsif mstart_rgd2'event and mstart_rgd2 = '1' then
           rdbusy_int <= '0'; 
    end if;
    if mnext'event  and mnext = '1' and next_read_clock = true then
        get_next_match <= '1';
    elsif mnext'event and mnext = '0' then
        get_next_match <= '0';
    end if;
end process read_control;

end;






-- altclklock megafunction 
-- Restrictions: 
--   1. NORMAL operation mode only 
--   2. Zero delay outputs
--   3. Locked output remains high for the entire simulation time 
--      after it is locked, and goes low when inclocken is deasserted.
--      invalid_lock_cycles parameter is not used for behavioral simulation
--
library ieee;
use ieee.std_logic_1164.all;

entity altclklock is
   generic (
            inclock_period : natural;
            inclock_settings : string := "UNUSED";
            valid_lock_cycles : natural := 3;
            invalid_lock_cycles : natural := 3;
            valid_lock_multiplier : natural := 1;
            invalid_lock_multiplier : natural := 1;
            operation_mode : string := "NORMAL";
            clock0_boost : natural := 1;
            clock0_divide : natural := 1;
            clock1_boost : natural := 1;
            clock1_divide : natural := 1;
            clock0_settings : string := "UNUSED";
            clock1_settings : string := "UNUSED";
            outclock_phase_shift : natural := 0);
	port (inclock : in std_logic;
         inclocken : in std_logic := '1';
         fbin : in std_logic := '0';
         clock0 : out std_logic;
         clock1: out std_logic;
         locked: out std_logic);
end altclklock;

architecture behavior of altclklock is

  signal pll0_period : TIME ;
  signal pll1_period : TIME ;
  signal new_clock0 : std_logic ;
  signal new_clock1 : std_logic ;
  signal start_new_clock0 : std_logic;
  signal start_new_clock1 : std_logic;
  signal phase_delay0 : TIME;
  signal phase_delay1 : TIME;
  signal locked_int : std_logic := '0';
  signal highcycle, lowcycle: time := 0 ps;
  signal cycleviolation : std_logic := '0';

begin


lock: process(inclock, inclocken)

variable inclock_edge_count: integer := 0;
variable outclock_start_edge : std_logic ;
begin

   if locked_int = '0' then
      pll0_period <= real(real(real(clock0_divide)/real(clock0_boost))* real(inclock_period )) * 1 ps;
      pll1_period <= real(real(real(clock1_divide)/real(clock1_boost))* real(inclock_period )) * 1 ps;

      if outclock_phase_shift = 0 then
         outclock_start_edge := '1'; 
         phase_delay0 <= (0.5 ) * pll0_period;
         phase_delay1 <= (0.5 ) * pll1_period;
      else
        phase_delay0 <= outclock_phase_shift * 1 ps;
        phase_delay1 <= outclock_phase_shift * 1 ps;
      end if;

        start_new_clock0 <= outclock_start_edge;
        start_new_clock1 <= outclock_start_edge;
   end if;
      
   if inclocken = '0' or cycleviolation = '1' then
      inclock_edge_count := 0;
      locked_int <= '0';
      locked <= '0';
   elsif inclock'event then -- any inclock edge
      if inclock = '1' then
         if ((NOW - lowcycle) /= (1 ps * inclock_period/2)) then
            cycleviolation <= '1';
	    assert false
            report "Duty cycle violation"
            severity error; 
         end if;
         highcycle <= NOW;
      end if;
      if inclock = '0' then
         if ((NOW - highcycle) /= (1 ps * inclock_period/2)) then
            cycleviolation <= '1';
	    assert false
            report "Duty cycle violation"
            severity error; 
         end if;
         lowcycle <= NOW;
      end if;
      inclock_edge_count := inclock_edge_count + 1; 
      if inclock_edge_count = valid_lock_cycles then
         cycleviolation <= '0';
         locked_int <= '1';
         locked <= '1';
      end if;
   end if;

end process;

gen_clock0: process(new_clock0,locked_int)
variable first_cycle: boolean := true;
begin
      if locked_int = '1' then
         if first_cycle = true then
            clock0 <= start_new_clock0;
            new_clock0 <= not start_new_clock0 after phase_delay0;
         else
            clock0 <= new_clock0; 
            new_clock0 <= not new_clock0 after pll0_period/2;
         end if;
         first_cycle := false;
      else
         first_cycle := true;
      end if;
end process;

gen_clock1: process(new_clock1,locked_int)
variable first_cycle: boolean := true;
begin
      if locked_int = '1' then
         if first_cycle = true then
            clock1 <= start_new_clock1;
            new_clock1 <= not start_new_clock1 after phase_delay1;
         else
            clock1 <= new_clock1; 
            new_clock1 <= not new_clock1 after pll1_period/2;
         end if;
         first_cycle := false;
      else
         first_cycle := true;
      end if;
end process;

          
end behavior;

-- lvds receiver megafunction
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity altlvds_rx is
    generic(
            number_of_channels: natural;
            deserialization_factor: natural;
            registered_output: string := "ON";
            inclock_period: natural;
            clock_setting: string := "UNUSED");
    port (
          rx_in: in std_logic_vector(number_of_channels-1 downto 0);
 			 rx_inclock: in std_logic;
			 rx_deskew: in std_logic := '0';
			 rx_out: out std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
          rx_outclock : out std_logic;
          rx_locked : out std_logic);
end altlvds_rx;

architecture behavior of altlvds_rx is
type channel_cnt is array (number_of_channels-1 downto 0) of integer;
signal data_int2, data_int3, data_regs: std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
signal rx_clkout_int : std_logic;
signal rx_locked_int : std_logic;
signal deskew_done: std_logic_vector(number_of_channels-1 downto 0) := (others => '1');
signal en: std_logic := '1';
signal first_load : std_logic := '0';
component altclklock
   generic (
            inclock_period : natural;
            inclock_settings : string := "UNUSED";
            valid_lock_cycles : natural := 3;
            invalid_lock_cycles : natural := 3;
            operation_mode : string := "NORMAL";
            clock0_boost : natural := 1;
            clock0_divide : natural := 1;
            clock1_boost : natural := 1;
            clock1_divide : natural := 1;
            clock0_settings : string := "UNUSED";
            clock1_settings : string := "UNUSED";
            outclock_phase_shift : natural := 0);
	port (inclock : in std_logic;
         inclocken : in std_logic;
         fbin : in std_logic := '0';
         clock0 : out std_logic;
         clock1: out std_logic;
         locked: out std_logic);
end component;
begin

rx_out    <= data_regs when registered_output = "ON" else data_int2;
rx_outclock <= rx_clkout_int;
rx_locked <= rx_locked_int;

U0: altclklock generic map (inclock_period => inclock_period,
                        clock0_boost => deserialization_factor)
           port map (inclock => rx_inclock, inclocken => en,
                     clock0 => rx_clkout_int, locked => rx_locked_int);

-- At start up deskew is done due to inability to detect unconnected rx_deskew
-- pin.
deskew: process(rx_deskew)
begin
   	if rx_deskew = '1' then
	   deskew_done <= (others => '0');
  	end if;
end process;


load_data: process(rx_clkout_int, rx_inclock)
variable rxin_cnt : integer := 0;
variable edge_count: integer := 0;
variable count: channel_cnt := (others => 0);
variable sample: integer;
variable start_data : integer := 0;
variable pattern, data_int: std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
begin
    if rx_inclock'event and rx_inclock = '1' then
      rxin_cnt := 0;
      if (deserialization_factor = 8)  then
         if (rx_deskew = '1') then
           for i in 0 to number_of_channels-1 loop
             if (pattern(((deserialization_factor*(i+1))-1) downto deserialization_factor*(i)) = "00001111") then
               count(i) := count(i) + 1;
             else
               count(i) := 0;
             end if;
             if count(i) = 3 then
                deskew_done(i) <= '1';
             end if;
           end loop;
         end if;
      elsif (deserialization_factor= 7) then
         if (rx_deskew = '1') then
           for i in 0 to number_of_channels-1 loop
             if (pattern(((deserialization_factor*(i+1))-1) downto deserialization_factor*(i)) = "0000111") then
               count(i) := count(i) + 1;
             else
               count(i) := 0;
             end if;
             if count(i) = 3 then
                deskew_done(i) <= '1';
             end if;
           end loop;
         end if;
      elsif (deserialization_factor= 4) then
         if (rx_deskew = '1') then
           for i in 0 to number_of_channels-1 loop
             if (pattern(((deserialization_factor*(i+1))-1) downto deserialization_factor*(i)) = "0011") then
               count(i) := count(i) + 1;
             else
               count(i) := 0;
             end if;
             if count(i) = 3 then
                deskew_done(i) <= '1';
             end if;
           end loop;
         end if;
      else
        deskew_done <= (others => '1');
      end if;
    elsif rx_clkout_int'event and rx_clkout_int = '0' then
       rxin_cnt := rxin_cnt + 1;
       if rxin_cnt = 3 then
         data_int2 <= data_int;
         start_data := 1;
      end if;
      if start_data = 1 then
       sample := edge_count rem deserialization_factor;
       edge_count := edge_count + 1;
       for i in 0 to number_of_channels -1 loop
         if deskew_done(i) = '1' then
           data_int((i*deserialization_factor)+sample) := rx_in(i);
         else
           pattern((i*deserialization_factor)+sample) := rx_in(i);
           data_int((i*deserialization_factor)+sample) := 'X';
         end if;
       end loop;
     end if;
    end if;
end process;


parallel_reg: process(rx_inclock)
begin
    if rx_inclock = '1' then
      if deserialization_factor = 8 or deserialization_factor = 7 then
        data_regs <= data_int2;
      else
        data_regs <= data_int3;
      end if;
    elsif rx_inclock = '0' then
      if deserialization_factor <= 4 then
        data_int3 <= data_int2;
      end if;
    end if;
end process parallel_reg;

end behavior;

-- lvds transmitter megafunction
--
    
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity altlvds_tx is
    generic(
            number_of_channels: natural;
            deserialization_factor: natural;
            registered_input: string := "ON";
            multi_clock: string := "OFF";
            inclock_period: natural;
            clock_setting: string := "UNUSED");
    port (
			 tx_in: in std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
 			 tx_inclock: in std_logic;
          sync_inclock: in std_logic := '0';
          tx_out: out std_logic_vector(number_of_channels-1 downto 0);
          tx_outclock : out std_logic;
          tx_locked : out std_logic);
end altlvds_tx;

architecture behavior of altlvds_tx is
signal data_int, data_reg, tx_in_rgd: std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
signal clock_in: std_logic;
signal clock_out: std_logic; 
signal tx_locked_int: std_logic; 
signal en: std_logic := '1';
component altclklock
   generic (
            inclock_period : natural;
            inclock_settings : string := "UNUSED";
            valid_lock_cycles : natural := 3;
            invalid_lock_cycles : natural := 3;
            operation_mode : string := "NORMAL";
            clock0_boost : natural := 1;
            clock0_divide : natural := 1;
            clock1_boost : natural := 1;
            clock1_divide : natural := 1;
            clock0_settings : string := "UNUSED";
            clock1_settings : string := "UNUSED";
            outclock_phase_shift : natural := 0);
	port (inclock : in std_logic;
         inclocken : in std_logic;
         fbin : in std_logic := '0';
         clock0 : out std_logic;
         clock1: out std_logic;
         locked: out std_logic);
end component;
begin

data_int  <= data_reg when registered_input = "ON" else tx_in;
clock_in <= sync_inclock when multi_clock = "ON" else tx_inclock;
tx_outclock <= clock_out;
tx_locked <= tx_locked_int;

U0: altclklock generic map (inclock_period => inclock_period,
                     clock0_boost => deserialization_factor)
           port map (inclock => tx_inclock, inclocken => en,
                     clock0 => clock_out, locked => tx_locked_int);

serialize: process(clock_out, clock_in)
variable edge_count: integer;
variable load_data : std_logic := '0';
variable count: integer := 0;
variable sample: integer;
variable tx_in2: std_logic_vector(deserialization_factor*number_of_channels -1 downto 0);
begin
    if clock_in'event and clock_in = '1' then
       edge_count := 0;
    end if;
    if clock_out'event and clock_out = '1' then
       edge_count := edge_count + 1;
       if edge_count = 3 then
          tx_in2 := data_int; -- register the incoming data on the third edge
          count := 0;
          load_data := '1'; -- third_rising edge
       end if;
       if load_data = '1' then
         for i in 0 to number_of_channels-1 loop
           tx_out(i) <= tx_in2(i*deserialization_factor + count);
         end loop;
         count := count + 1;
       end if;
    end if;
end process;

sync: process(clock_in)
begin
    if clock_in = '0' then
      if deserialization_factor = 8 or deserialization_factor = 7 then
        data_reg <= tx_in_rgd;
      end if;
    elsif clock_in = '1' then
      if deserialization_factor <= 4 then
        data_reg <= tx_in;
      end if;
    end if;
end process sync;

end behavior;























